//
//  AMHelpViewController.m
//  HelpViewer
//
//  Created by Andreas on 03.04.11.
//
//	This was mostly taken from Apple's PageControl sample code


#import "AMHelpViewController.h"
#import "AMHelpPageController.h"
#import "UIDevice_AMAdditions.h"


//	Info.plist keys:

static NSString *const kAMHelpFolderName = @"AMHelpFolderName";
static NSString *const kAMHelpFolderName_Landscape = @"AMHelpFolderName-Landscape";
static NSString *const kAMHelpFolderName_Pad = @"AMHelpFolderName-iPad";
static NSString *const kAMHelpFolderName_Pad_Landscape = @"AMHelpFolderName-iPad-Landscape";
static NSString *const kAMHelpPageCount = @"AMHelpPageCount";
static NSString *const kAMHelpPageCount_Pad = @"AMHelpPageCount-iPad";
	

@interface AMHelpViewController ()
- (AMHelpPageController *)pageControllerForPage:(int)page;
- (void)initFolderNamesFromInfoDictionary;
- (NSInteger)pageCountFromInfoDictionary;
- (CGRect)frameForPage:(int)page;
- (void)loadScrollViewWithPage:(int)page;
- (void)scrollViewDidScroll:(UIScrollView *)sender;
@end


@implementation AMHelpViewController

@synthesize pageControl;
@synthesize scrollView;
@synthesize headerItem;
@synthesize viewControllers;
@synthesize folderName;
@synthesize folderNameLandscape;
@synthesize pageCount;


- (void)_commonInit
{
	// a page is the width of the scroll view
	scrollView.pagingEnabled = YES;
	scrollView.showsHorizontalScrollIndicator = NO;
	scrollView.showsVerticalScrollIndicator = NO;
	scrollView.directionalLockEnabled = YES;
	scrollView.scrollsToTop = NO;
	scrollView.delegate = self;
	
	pageControl.numberOfPages = pageCount;
	pageControl.currentPage = 0;
}

- (id)initWithFolderName:(NSString *)folderPortrait folderNameLandscape:(NSString *)folderLandscape pageCount:(NSInteger)pages;
{
	self = [super initWithNibName:@"HelpView" bundle:nil];
	if (self != nil) {
		self.folderName = folderPortrait;
		self.folderNameLandscape = folderLandscape;
		self.pageCount = pages;
		[self _commonInit];
	}
	return self;
}

- (id)initWithDataFromInfoDictionary
{
	NSInteger pages = [self pageCountFromInfoDictionary];
	if ((self = [self initWithFolderName:nil folderNameLandscape:nil pageCount:pages])) {
		[self initFolderNamesFromInfoDictionary];
	}
	return self;
}

- (void)awakeFromNib
{
	self.pageCount = [self pageCountFromInfoDictionary];
	[self initFolderNamesFromInfoDictionary];
	[self _commonInit];
}

- (void)dealloc
{
	[folderName release];
	[pageControl release];
	[scrollView release];
	[headerItem release];
	[viewControllers release];
	[super dealloc];
}

- (void)setTitle:(NSString *)newTitle
{
	headerItem.title = newTitle;
}

- (NSString *)title
{
	return headerItem.title;
}


// -----------------------------------------------------------------------------
#pragma mark public
// -----------------------------------------------------------------------------

- (IBAction)changePage:(id)sender
{
    NSInteger page = pageControl.currentPage;
	
    // load the visible page and the page on either side of it (to avoid flashes when the user starts scrolling)
    [self loadScrollViewWithPage:page - 1];
    [self loadScrollViewWithPage:page];
    [self loadScrollViewWithPage:page + 1];
    
	// update the scroll view to the appropriate page
    [scrollView scrollRectToVisible:[self frameForPage:page] animated:YES];
    
	// page control was being used; we do NOT need to adjust the page control's current page when the scroll view moves
    adjustPageNumber = NO;
}

- (NSString *)folderNameForCurrentInterfaceOrientation
{
	NSString *result = nil;
	if (UIInterfaceOrientationIsLandscape(self.interfaceOrientation)) {
		result = self.folderNameLandscape;
	}
	if (!result) {
		result = self.folderName;
	}
	return result;
}


// -----------------------------------------------------------------------------
#pragma mark private
// -----------------------------------------------------------------------------

- (void)initFolderNamesFromInfoDictionary
{
	// get help folder name depending on device and interface orientation
	if ([[UIDevice currentDevice] isPad]) {
		// iPad
		self.folderName = [[NSBundle mainBundle] objectForInfoDictionaryKey:kAMHelpFolderName_Pad];
		self.folderNameLandscape = [[NSBundle mainBundle] objectForInfoDictionaryKey:kAMHelpFolderName_Pad_Landscape];
	} else {
		// iPhone
		self.folderName = [[NSBundle mainBundle] objectForInfoDictionaryKey:kAMHelpFolderName];
		self.folderNameLandscape = [[NSBundle mainBundle] objectForInfoDictionaryKey:kAMHelpFolderName_Landscape];
	}
}

- (NSInteger)pageCountFromInfoDictionary
{
	NSInteger result = 0;
	// get help page count depending on device
	// (the page count should be identical for both interface orientations)
	NSNumber *numberOfPages = nil;
	if ([[UIDevice currentDevice] isPad]) {
		// iPad
		numberOfPages = [[NSBundle mainBundle] objectForInfoDictionaryKey:kAMHelpPageCount_Pad];
	} else {
		numberOfPages = [[NSBundle mainBundle] objectForInfoDictionaryKey:kAMHelpPageCount];
	}
	result = [numberOfPages integerValue];
	return result;
}


- (void)loadScrollViewWithPage:(int)page
{
    if (page < 0) return;
    if (page >= pageCount) return;
	
    // replace the placeholder if necessary
    AMHelpPageController *controller = [self pageControllerForPage:page];
	
    // add the controller's view to the scroll view
    if (!controller.view.superview) {
        controller.view.frame = [self frameForPage:page];
        [scrollView addSubview:controller.view];
    }
	controller.view.hidden = NO;
}

- (NSMutableArray *)viewControllers
{
	NSMutableArray *result = viewControllers;
	if (!viewControllers) {
		self.viewControllers = [NSMutableArray array];
		// load the array with placeholders which will be replaced on demand
		for (unsigned i = 0; i < pageCount; i++) {
			[viewControllers addObject:[NSNull null]];
		}
	}
	return result;
}

- (AMHelpPageController *)pageControllerForPage:(int)page
{
	// make sure viewControllers array is initialized
	(void)[self viewControllers];
	AMHelpPageController *result = nil;
	result = [viewControllers objectAtIndex:page];
	if ((id)result == [NSNull null]) {
		result = [[AMHelpPageController alloc] initWithController:self pageNumber:page];
		[viewControllers replaceObjectAtIndex:page withObject:result];
		[result release];
	}
	return result;
}

- (CGRect)frameForPage:(int)page
{
    CGRect result = scrollView.frame;
    result.origin.x = result.size.width * page;
    result.origin.y = 0;
	return result;
}


// -----------------------------------------------------------------------------
#pragma mark view controller
// -----------------------------------------------------------------------------

- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	
	scrollView.contentSize = CGSizeMake(scrollView.frame.size.width * pageCount, scrollView.frame.size.height);
	// pages are created on demand
	// load the visible page
	// load the page on either side to avoid flashes when the user starts scrolling
	[self loadScrollViewWithPage:0];
	[self loadScrollViewWithPage:1];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
//	BOOL result = [[UIDevice currentDevice] isPad] || UIInterfaceOrientationIsPortrait(interfaceOrientation);
//	NSLog(@"shouldAutorotateToInterfaceOrientation: - requested orientation %@", (UIInterfaceOrientationIsPortrait(interfaceOrientation) ? @"is portrait" : @"is not portrait"));
//	NSLog(@"result is %@", (result ? @"YES" : @"NO"));
//	return result;
	return YES;
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
	// hide non-active pages so they will not show up during rotation
	for (int page = 0; page < pageCount; page++) {
		AMHelpPageController *controller = [self pageControllerForPage:page];
		if (page != pageControl.currentPage) {
			controller.view.hidden = YES;
		}
	}
	// we do NOT want to adjust the page control's current page when the scroll view moves
    adjustPageNumber = NO;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
	BOOL needsReload = ![self.folderName isEqualToString:self.folderNameLandscape];
	// fix content size
	scrollView.contentSize = CGSizeMake(scrollView.frame.size.width * pageCount, scrollView.frame.size.height);
	// we need to re-layout the help pages
	for (int page = 0; page < pageCount; page++) {
		AMHelpPageController *controller = [self pageControllerForPage:page];
        controller.view.frame = [self frameForPage:page];
		// in case we have different folders for portrait and landscape
		// we need to reload the page contents
		if (needsReload) {
			[controller reloadData];
		}
	}
	// adjust offset
	[scrollView scrollRectToVisible:[self frameForPage:pageControl.currentPage] animated:NO];
}


// -----------------------------------------------------------------------------
#pragma mark scroll view delegate
// -----------------------------------------------------------------------------

- (void)scrollViewDidScroll:(UIScrollView *)sender
{
    // We don't want a "feedback loop" between the UIPageControl and the scroll delegate in
    // which a scroll event generated from the user hitting the page control triggers updates from
    // the delegate method. We use a boolean to disable the delegate logic when the page control is used.
    if (adjustPageNumber) {
        // the scroll was initiated from the scroll view - do adjust page number of page control
		// Switch the indicator when more than 50% of the previous/next page is visible
		CGFloat pageWidth = scrollView.frame.size.width;
		NSInteger page = floor((scrollView.contentOffset.x - pageWidth / 2) / pageWidth) + 1;
		pageControl.currentPage = page;
		
		// load the visible page and the page on either side of it (to avoid flashes when the user starts scrolling)
		[self loadScrollViewWithPage:page - 1];
		[self loadScrollViewWithPage:page];
		[self loadScrollViewWithPage:page + 1];
		
		// A possible optimization would be to unload the views+controllers which are no longer visible
		
		self.title = [[self pageControllerForPage:page] pageTitle];
	}
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView 
{
	// while scroll dragging, we need to adjust the page control's current page
    adjustPageNumber = YES;
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
	// while scroll dragging, we need to adjust the page control's current page
    adjustPageNumber = YES;
}


@end



@implementation AMHelpViewController (AMHelpPageDelegate)

- (void)pageDidFinishLoad:(AMHelpPageController *)helpPageController
{
	if (helpPageController.pageNumber == pageControl.currentPage) {
		self.title = helpPageController.pageTitle;
	}
}


@end


